<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2016 IMIA net based solutions (info@imia.de)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace IMIA\ImiaBaseExt\Domain\Repository;

use IMIA\ImiaBaseExt\Xclass\Extbase\Persistence\Generic\Typo3QuerySettings;
use TYPO3\CMS\Core\Database\DatabaseConnection;
use TYPO3\CMS\Extbase\Persistence\Generic\Mapper\DataMap;
use TYPO3\CMS\Extbase\Persistence\Generic\Query;
use TYPO3\CMS\Extbase\Persistence\QueryInterface;
use TYPO3\CMS\Extbase\Persistence\QueryResultInterface;

/**
 * @package     imia_base_ext
 * @subpackage  Domain\Repository
 * @author      David Frerich <d.frerich@imia.de>
 */
class Repository extends \TYPO3\CMS\Extbase\Persistence\Repository
{
    /**
     * @var array
     */
    protected $defaultOrderings = [
        'sorting' => QueryInterface::ORDER_ASCENDING,
    ];

    /**
     * @var boolean
     *
     * @deprecated
     */
    protected $respectEnableFields = true;

    /**
     * @var boolean
     */
    protected $respectStoragePage = true;

    /**
     * @var boolean
     */
    protected $respectSysLanguage = true;

    /**
     * @var boolean
     */
    protected $ignoreEnableFields = false;

    /**
     * @var boolean
     */
    protected $includeDeleted = false;

    /**
     * @var mixed
     */
    protected $languageOverlayMode = null;

    /**
     * @var string
     */
    protected $languageMode = null;

    /**
     * @var integer
     */
    protected $languageUid = null;

    /**
     * @var array
     */
    protected $enableFieldsToBeIgnored = null;

    /**
     * @var array
     */
    protected $storagePageIds = null;

    /**
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return QueryResultInterface
     */
    public function findAll($limit = null, $offset = null, $sorting = null)
    {
        return $this->createQuery($limit, $offset, $sorting)->execute();
    }

    /**
     * @param array $uids
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return QueryResultInterface
     */
    public function findByUids(array $uids, $limit = null, $offset = null, $sorting = null)
    {
        if (count($uids) > 0) {
            $query = $this->createQuery($limit, $offset, $sorting);
            $constraints = [
                $query->in('uid', $uids),
            ];

            $transPointerField = $GLOBALS['TCA'][$this->getTableName()]['ctrl']['transOrigPointerField'];
            if ($transPointerField) {
                $constraints[] = $query->in($transPointerField, $uids);
            }

            $query->matching($query->logicalOr($constraints));

            return $query->execute();
        } else {
            return [];
        }
    }

    /**
     * @param array $uids
     * @return QueryResultInterface
     */
    public function findByInUid(array $uids)
    {
        $query = $this->createQuery();
        $constraints = [
            $query->in('uid', $uids),
        ];

        $transPointerField = $GLOBALS['TCA'][$this->getTableName()]['ctrl']['transOrigPointerField'];
        if ($transPointerField) {
            $constraints[] = $query->in($transPointerField, $uids);
        }

        $query->matching($query->logicalOr($constraints));

        return $query->execute();
    }

    /**
     * @param integer $pid
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return QueryResultInterface
     */
    public function findByPid($pid, $limit = null, $offset = null, $sorting = null)
    {
        $query = $this->createQuery($limit, $offset, $sorting);
        $query->matching(
            $query->equals('pid', $pid)
        );

        return $query->execute();
    }

    /**
     * @return integer
     */
    public function findLastModified()
    {
        $query = $this->createQuery(1, 0, ['tstamp', 'desc']);
        $result = $this->executeArrayQuery($query, ['tstamp']);

        $lastModified = 0;
        foreach ($result as $row) {
            $lastModified = $row['tstamp'];
            break;
        }

        return $lastModified;
    }

    /**
     * @param integer $limit
     * @param integer $offset
     * @param array $sorting
     * @return Query
     */
    public function createQuery($limit = null, $offset = null, $sorting = null)
    {
        $this->getTableName(); // fix wrong datamap
        $query = parent::createQuery();

        if (!$this->respectEnableFields || $this->ignoreEnableFields) {
            $query->getQuerySettings()->setIgnoreEnableFields(true);
        }
        if (!$this->respectEnableFields || $this->includeDeleted) {
            $query->getQuerySettings()->setIncludeDeleted(true);
        }
        if (!$this->respectStoragePage) {
            $query->getQuerySettings()->setRespectStoragePage(false);
        }
        if (!$this->respectSysLanguage) {
            $query->getQuerySettings()->setRespectSysLanguage(false);
        }
        if ($this->languageOverlayMode !== null) {
            $query->getQuerySettings()->setLanguageOverlayMode($this->languageOverlayMode);
        }
        if ($this->languageMode !== null) {
            $query->getQuerySettings()->setLanguageMode($this->languageMode);
        }
        if ($this->languageUid !== null) {
            $query->getQuerySettings()->setLanguageUid($this->languageUid);
        }
        if ($this->enableFieldsToBeIgnored !== null) {
            $query->getQuerySettings()->setEnableFieldsToBeIgnored($this->enableFieldsToBeIgnored);
        }
        if ($this->storagePageIds !== null) {
            $query->getQuerySettings()->setStoragePageIds($this->storagePageIds);
        }

        if ($limit && $limit > 0) {
            $query->setLimit($limit);
        }
        if ($offset && $offset > 0) {
            $query->setOffset($offset);
        }
        if ($sorting && is_array($sorting) && $sorting[0]) {
            $query->setOrderings([
                $sorting[0] => strtolower($sorting[1]) == 'asc' ?
                    QueryInterface::ORDER_ASCENDING :
                    QueryInterface::ORDER_DESCENDING,
            ]);
        }

        return $query;
    }

    /**
     * @param boolean $respectEnableFields
     * @return \IMIA\ImiaBaseExt\Domain\Repository\Repository
     *
     * @deprecated
     */
    public function setRespectEnableFields($respectEnableFields)
    {
        $this->setIgnoreEnableFields(!$respectEnableFields);
        $this->setIncludeDeleted(!$respectEnableFields);

        return $this;
    }

    /**
     * @param QueryInterface $query
     * @return array
     */
    protected function executeUidArrayQuery($query)
    {
        $uids = [];
        $result = $this->executeArrayQuery($query, ['uid']);
        foreach ($result as $row) {
            $uids[] = $row['uid'];
        }

        return $uids;
    }

    /**
     * @param QueryInterface $query
     * @param array $fields
     * @return array
     */
    protected function executeArrayQuery($query, array $fields = ['uid'])
    {
        /** @var Typo3QuerySettings $querySettings */
        $querySettings = $query->getQuerySettings();
        $querySettings->setOnlyFields($fields);

        return $query->execute(true);
    }

    /**
     * @param \TYPO3\CMS\Extbase\Persistence\QueryInterface $query
     * @param array $parameters
     * @return string
     */
    protected function getQuerySql($query, &$parameters = null)
    {
        if (!$parameters) {
            $parameters = [];
        }

        $storageBackend = $this->objectManager->get('TYPO3\CMS\Extbase\Persistence\Generic\Storage\Typo3DbBackend');
        $statementParts = $storageBackend->parseQuery($query, $parameters);

        return $storageBackend->buildQuery($statementParts, $parameters);
    }

    /**
     * @inheritdoc
     */
    public function __call($methodName, $arguments)
    {
        if (substr($methodName, 0, 8) === 'findByIn' && strlen($methodName) > 8 && ctype_upper(substr($methodName, 8, 1))) {
            $propertyName = lcfirst(substr($methodName, 8));
            $query = $this->createQuery();
            $query->matching($query->in($propertyName, $arguments[0]));

            return $query->execute();
        } elseif (substr($methodName, 0, 11) === 'findOneByIn' && strlen($methodName) > 11 && ctype_upper(substr($methodName, 11, 1))) {
            $propertyName = lcfirst(substr($methodName, 11));
            $query = $this->createQuery();
            $query->matching($query->in($propertyName, $arguments[0]));

            return $query->execute()->current();
        } elseif (substr($methodName, 0, 12) === 'findUidsByIn' && strlen($methodName) > 12 && ctype_upper(substr($methodName, 12, 1))) {
            $propertyName = lcfirst(substr($methodName, 12));
            $query = $this->createQuery();
            $query->matching($query->in($propertyName, $arguments[0]));

            return $this->executeUidArrayQuery($query);
        } elseif (substr($methodName, 0, 10) === 'findUidsBy' && strlen($methodName) > 10 && ctype_upper(substr($methodName, 10, 1))) {
            $propertyName = lcfirst(substr($methodName, 10));
            $query = $this->createQuery();
            $query->matching($query->in($propertyName, $arguments[0]));

            return $this->executeUidArrayQuery($query);
        } else {
            return parent::__call($methodName, $arguments);
        }
    }

    public function truncate()
    {
        $this->getDb()->exec_TRUNCATEquery($this->getTableName());

        return true;
    }

    /**
     * @return string
     */
    public function getTableName()
    {
        /** @var DataMap $dataMap */
        $dataMap = $this->objectManager
            ->get('TYPO3\CMS\Extbase\Persistence\Generic\Mapper\DataMapper')
            ->getDataMap($this->objectType);

        if ($GLOBALS['TSFE'] && isset($GLOBALS['TSFE']->tmpl->setup['plugin.'])) {
            $className = get_class($this);
            if (strpos($className, '\\') !== false) {
                $classNameParts = explode('\\', $className, 4);
                // Skip vendor and product name for core classes
                if (strpos($className, 'TYPO3\\CMS\\') === 0) {
                    $extensionName = $classNameParts[2];
                } else {
                    $extensionName = $classNameParts[1];
                }
            } else {
                list(, $extensionName) = explode('_', $className);
            }

            $tsExtName = 'tx_' . strtolower($extensionName);
            $tsTableName = $GLOBALS['TSFE']->tmpl->setup['plugin.'][$tsExtName . '.']['persistence.']['classes.'][$dataMap->getClassName() . '.']['mapping.']['tableName'];

            if (isset($tsTableName) && $tsTableName != $dataMap->getTableName()) {
                $dataMap->setTableName($tsTableName);
            }
        }

        return $dataMap->getTableName();
    }

    /**
     * @return array
     */
    public function getStoragePageIds()
    {
        return $this->storagePageIds;
    }

    /**
     * @param array $storagePageIds
     * @return $this
     */
    public function setStoragePageIds($storagePageIds)
    {
        $this->storagePageIds = $storagePageIds;

        return $this;
    }

    /**
     * @return int
     */
    public function getLanguageUid()
    {
        return $this->languageUid;
    }

    /**
     * @param int $languageUid
     * @return $this
     */
    public function setLanguageUid($languageUid)
    {
        $this->languageUid = $languageUid;

        return $this;
    }

    /**
     * @return string
     */
    public function getLanguageMode()
    {
        return $this->languageMode;
    }

    /**
     * @param string $languageMode
     * @return $this
     */
    public function setLanguageMode($languageMode)
    {
        $this->languageMode = $languageMode;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getLanguageOverlayMode()
    {
        return $this->languageOverlayMode;
    }

    /**
     * @param mixed $languageOverlayMode
     * @return $this
     */
    public function setLanguageOverlayMode($languageOverlayMode)
    {
        $this->languageOverlayMode = $languageOverlayMode;

        return $this;
    }

    /**
     * @return boolean
     */
    public function getIncludeDeleted()
    {
        return $this->includeDeleted;
    }

    /**
     * @param boolean $includeDeleted
     * @return $this
     */
    public function setIncludeDeleted($includeDeleted)
    {
        $this->includeDeleted = $includeDeleted;

        return $this;
    }

    /**
     * @return boolean
     */
    public function getIgnoreEnableFields()
    {
        return $this->ignoreEnableFields;
    }

    /**
     * @param boolean $ignoreEnableFields
     * @return $this
     */
    public function setIgnoreEnableFields($ignoreEnableFields)
    {
        $this->ignoreEnableFields = $ignoreEnableFields;

        return $this;
    }

    /**
     * @return boolean
     */
    public function getRespectSysLanguage()
    {
        return $this->respectSysLanguage;
    }

    /**
     * @param boolean $respectSysLanguage
     * @return $this
     */
    public function setRespectSysLanguage($respectSysLanguage)
    {
        $this->respectSysLanguage = $respectSysLanguage;

        return $this;
    }

    /**
     * @return boolean
     */
    public function getRespectStoragePage()
    {
        return $this->respectStoragePage;
    }

    /**
     * @param boolean $respectStoragePage
     * @return $this
     */
    public function setRespectStoragePage($respectStoragePage)
    {
        $this->respectStoragePage = $respectStoragePage;

        return $this;
    }

    /**
     * @return array
     */
    public function getEnableFieldsToBeIgnored()
    {
        return $this->enableFieldsToBeIgnored;
    }

    /**
     * @param array $enableFieldsToBeIgnored
     * @return $this
     */
    public function setEnableFieldsToBeIgnored($enableFieldsToBeIgnored)
    {
        $this->enableFieldsToBeIgnored = $enableFieldsToBeIgnored;

        return $this;
    }

    /**
     * @return DatabaseConnection
     */
    protected function getDb()
    {
        return $GLOBALS['TYPO3_DB'];
    }
}