<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2016 IMIA net based solutions (info@imia.de)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace IMIA\ImiaBaseExt\Xclass\Backend\Form\FormDataProvider;

use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * @package     imia_base_ext
 * @subpackage  Xclass
 * @author      David Frerich <d.frerich@imia.de>
 */
class EvaluateDisplayConditions extends \TYPO3\CMS\Backend\Form\FormDataProvider\EvaluateDisplayConditions
{
    /**
     * @var string
     */
    protected $recordTable;

    /**
     * @inheritdoc
     */
    public function addData(array $result)
    {
        $this->recordTable = $result['tableName'];

        return parent::addData($result);
    }

    /**
     * @inheritdoc
     */
    protected function matchFieldCondition($condition, $record, $flexformContext = false)
    {
        list($fieldName, $operator, $operand) = explode(':', $condition, 3);
        if ($flexformContext) {
            if (strpos($fieldName, 'parentRec.') !== false) {
                $fieldNameParts = explode('.', $fieldName, 2);
                $fieldValue = $record['parentRec'][$fieldNameParts[1]];
            } else {
                $fieldValue = $record[$fieldName]['vDEF'];
            }
        } else {
            $fieldValue = $this->getRecordFieldValue($fieldName, $record);
        }

        $result = false;
        switch ($operator) {
            case 'REQ':
                if (is_array($fieldValue) && count($fieldValue) <= 1) {
                    $fieldValue = array_shift($fieldValue);
                }
                if (strtoupper($operand) === 'TRUE') {
                    $result = (bool)$fieldValue;
                } else {
                    $result = !$fieldValue;
                }
                break;
            case '>':
                if (is_array($fieldValue) && count($fieldValue) <= 1) {
                    $fieldValue = array_shift($fieldValue);
                }
                $result = $fieldValue > $operand;
                break;
            case '<':
                if (is_array($fieldValue) && count($fieldValue) <= 1) {
                    $fieldValue = array_shift($fieldValue);
                }
                $result = $fieldValue < $operand;
                break;
            case '>=':
                if (is_array($fieldValue) && count($fieldValue) <= 1) {
                    $fieldValue = array_shift($fieldValue);
                }
                $result = $fieldValue >= $operand;
                break;
            case '<=':
                if (is_array($fieldValue) && count($fieldValue) <= 1) {
                    $fieldValue = array_shift($fieldValue);
                }
                $result = $fieldValue <= $operand;
                break;
            case '-':
            case '!-':
                if (is_array($fieldValue) && count($fieldValue) <= 1) {
                    $fieldValue = array_shift($fieldValue);
                }
                list($minimum, $maximum) = explode('-', $operand);
                $result = $fieldValue >= $minimum && $fieldValue <= $maximum;
                if ($operator[0] === '!') {
                    $result = !$result;
                }
                break;
            case '=':
            case '!=':
                if (is_array($fieldValue) && count($fieldValue) <= 1) {
                    $fieldValue = array_shift($fieldValue);
                }
                $result = $fieldValue == $operand;
                if ($operator[0] === '!') {
                    $result = !$result;
                }
                break;
            case 'IN':
            case '!IN':
                if (is_array($fieldValue)) {
                    $result = count(array_intersect($fieldValue, explode(',', $operand))) > 0;
                } else {
                    $result = GeneralUtility::inList($operand, $fieldValue);
                }
                if ($operator[0] === '!') {
                    $result = !$result;
                }
                break;
            case 'BIT':
            case '!BIT':
                $result = (bool)((int)$fieldValue & $operand);
                if ($operator[0] === '!') {
                    $result = !$result;
                }
                break;
        }

        return $result;
    }

    /**
     * @param string $fieldName
     * @param array $record
     * @return string
     */
    protected function getRecordFieldValue($fieldName, $record)
    {
        if (strpos($fieldName, '.') !== false) {
            $fieldValue = null;

            $fieldNameParts = explode('.', $fieldName, 2);
            $parentFieldName = $fieldNameParts[0];
            $subFieldName = $fieldNameParts[1];

            if (array_key_exists($parentFieldName, $record) && $record[$parentFieldName]) {
                $recordFieldValue = $record[$parentFieldName];
                if (is_array($recordFieldValue)) {
                    $recordFieldValue = (int)implode(',', $recordFieldValue);
                } elseif (!is_numeric($recordFieldValue)) {
                    $arrValue = explode('|', $recordFieldValue);
                    $recordFieldValue = (int)preg_replace('/^.*?_([0-9]+)$/ism', '$1', $arrValue[0]);
                }

                $fieldTCA = $GLOBALS['TCA'][$this->recordTable]['columns'][$parentFieldName];
                if ($fieldTCA || $parentFieldName == 'pid') {
                    $subRecordTable = '';
                    $subRecordUid = 0;

                    if ($parentFieldName == 'pid') {
                        $subRecordTable = 'pages';
                        $subRecordUid = (int)$recordFieldValue;
                    } else {
                        switch ($fieldTCA['config']['type']) {
                            case 'select':
                            case 'inline':
                                if ($fieldTCA['config']['foreign_table'] && !$fieldTCA['config']['MM']) {
                                    $recordFieldValues = explode(',', $recordFieldValue);
                                    $subRecordTable = trim($fieldTCA['config']['foreign_table']);
                                    $subRecordUid = (int)array_shift($recordFieldValues);
                                }
                                break;
                            case 'group':
                                if ($fieldTCA['config']['internal_type'] == 'db' && $fieldTCA['config']['allowed']) {
                                    $recordFieldValues = explode(',', $recordFieldValue);
                                    $allowedTables = array_map('trim', explode(',', $fieldTCA['config']['allowed']));

                                    if (count($allowedTables) === 1 || is_numeric($recordFieldValues)) {
                                        $subRecordTable = array_shift($allowedTables);
                                    } else {
                                        foreach ($allowedTables as $table) {
                                            if (strpos($recordFieldValues[0], $table) === 0) {
                                                $subRecordTable = $table;
                                                break;
                                            }
                                        }
                                    }

                                    $subRecordUid = (int)array_shift(explode('|', str_replace($subRecordTable . '_', '', $recordFieldValue)));
                                }
                                break;
                            case 'inline':
                                break;
                        }
                    }

                    if ($subRecordTable && $subRecordUid) {
                        $res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', $subRecordTable, 'uid=' . $subRecordUid . BackendUtility::deleteClause($subRecordTable));
                        $subRecord = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);
                        $GLOBALS['TYPO3_DB']->sql_free_result($res);

                        if ($subRecord) {
                            BackendUtility::fixVersioningPid($subRecordTable, $subRecord);
                            $fieldValue = $this->getRecordFieldValue($subFieldName, $subRecord);
                        }
                    }
                }
            }
        } else {
            $fieldValue = $record[$fieldName];
        }

        return $fieldValue;
    }
}