<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2016 IMIA net based solutions (info@imia.de)
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

namespace IMIA\ImiaBaseExt\Xclass\Core\DataHandling;

use TYPO3\CMS\Backend\Utility\BackendUtility;

/**
 * @package     imia_base_ext
 * @subpackage  Xclass
 * @author      David Frerich <d.frerich@imia.de>
 */
class DataHandler extends \TYPO3\CMS\Core\DataHandling\DataHandler
{
    /**
     * @var integer
     */
    protected static $deletedCount = 0;

    /**
     * @inheritdoc
     */
    public function copySpecificPage($uid, $destPid, $copyTablesArray, $first = false)
    {
        $copyTablesArrayWithoutExcludes = [];
        foreach ($copyTablesArray as $table) {
            if (!isset($GLOBALS['TCA'][$table]['ctrl']['excludeOnPageCopy']) || !$GLOBALS['TCA'][$table]['ctrl']['excludeOnPageCopy']) {
                $copyTablesArrayWithoutExcludes[] = $table;
            }
        }

        return parent::copySpecificPage($uid, $destPid, $copyTablesArrayWithoutExcludes, $first);
    }

    /**
     * @inheritdoc
     */
    public function deleteRecord($table, $uid, $noRecordCheck = false, $forceHardDelete = false, $undeleteRecord = false)
    {
        self::$deletedCount++;
        if (self::$deletedCount == 100) {
            $errorMail = getenv('ERROR_MAIL') ?: ($_ENV['ERROR_MAIL'] ?: ($_SERVER['ERROR_MAIL'] ?: $_SERVER['REDIRECT_ERROR_MAIL']));

            if ($errorMail) {
                ob_start();
                debug_print_backtrace();
                $errorMessage = ob_get_contents();
                ob_end_clean();

                $domain = $_SERVER['HTTP_HOST'];
                mail($errorMail, 'TYPO3 Warning: DELETE Operation (> 100)' . ($domain ? ' | ' . $domain : ''), $errorMessage,
                    'From: dev@imia.de' . "\r\n" .
                    'Reply-To: dev@imia.de' . "\r\n" .
                    'X-Mailer: PHP/' . phpversion());
            }
        }

        if ($table == 'tt_content' && $uid) {
            $record = BackendUtility::getRecord($table, $uid);
            if ($record && $record['sorting'] < 10000000) {
                $GLOBALS['TYPO3_DB']->exec_UPDATEquery($table, 'uid = ' . $uid, [
                    'deleted_sorting' => $record['sorting'],
                ]);
            }
        }

        parent::deleteRecord($table, $uid, $noRecordCheck, $forceHardDelete, $undeleteRecord);
    }

    /**
     * @param string $table
     * @param string $uid
     * @param string $field
     * @param string $value
     * @param array $conf
     * @param bool $undeleteRecord
     */
    public function deleteRecord_procBasedOnFieldType($table, $uid, $field, $value, $conf, $undeleteRecord = false)
    {
        if ($table != 'tt_content' || $field != 'tx_flux_children' || $value !== null) {
            parent::deleteRecord_procBasedOnFieldType($table, $uid, $field, $value, $conf, $undeleteRecord);
        }
    }

    /**
     * @inheritdoc
     */
    public function moveRecord($table, $uid, $destPid)
    {
        if (in_array($table, ['tx_imiastatistics_pages'])) {
            return;
        }

        parent::moveRecord($table, $uid, $destPid);
    }
}
